"""
Small utility for me or your personal use

Usage:
------

    $ pylaxz -flag argument

Available flag are:
    -h, --help         Show this help
    -u, --update       Update available check
    -v, --version      Show current version

    -N, --network      Check network infomation
        Arguments : (ip), (i)nternet, (s)peed, 
    -S, --system       Check system information
        Arguments : (os), (os-info), 

    -L, --lxz      Ingegrate with lxz TODO
        --help for HELP.
        


More information is available at:
- https://pypi.org/project/py-laxz/
- https://github.com/minlaxz/py-laxz/

"""
# Standard library imports
from subprocess import run  # , PIPE, Popen, STDOUT
from sys import argv, stdout, stderr
import getopt
import os

# pylaxz imports
from .utils import logxs, _network, _system
from .__version__ import version


# save_history = False

DIR_PATH = os.path.dirname(os.path.realpath(__file__))
SCRIPT_PATH = os.path.join(DIR_PATH, "shells/")

SYSX_PATH = "bash " + SCRIPT_PATH + "sysx.sh "
HOWX_PATH = "bash " + SCRIPT_PATH + "howx.sh "

SCRIPTX_PATH = "bash " + SCRIPT_PATH + "scriptx.sh "

DKX_PATH = "bash " + SCRIPT_PATH + "dkx.sh "

LXZ_CHECK = ["--has-internet"]

LXZ_HOW = ["--how-enc", "--how-dec", "--how-compress", "--how-decompress",
            "--how-copy",  "--how-safe-rm", "--issue-opencv", "--how-find-mv"]

LXZ_SCRIPT = ["--lxz-DE", "--help", "--version", "--help-long"]


_LXZ_S_NETWORK = ["--scan-host", "--port-service"]
_LXZ_S_SYS = ["--sys-upgrade",
              "--sys-setup", "--is-installed"]
LXZ_SYS_ARG = _LXZ_S_NETWORK + _LXZ_S_SYS

# if save_history:
#     from .orm import Database
#     db = Database(DIR_PATH+'/data/sqlite.db')
# else:
#     db = None


# class DBModel(db.Model if db else None):
#     text = str
#     time = str
#     direct = bool

#     def __init__(self, text, time, direct) -> None:
#         self.text = text
#         self.time = time
#         self.direct = direct


def main(direct=True):
    """

    Main function of pylaxz

    """

    # if save_history:
    #     _save([argv[:], direct])

    if len(argv[1:]) == 0:
        logxs.printf("$ pylaxz -h for help", _int=True)

    try:
        args, _ = getopt.getopt(
            argv[1:],
            "hvuiN:S:L:",
            ["help", "version", "update", "info", "network", "system", "lxz"],
        )
        for c_flag, c_val in args:

            # stable args
            if c_flag in ("-h", "--help"):
                _logxs_out(__doc__)
            elif c_flag in ("-v", "--version"):
                _logxs_out(version)
            elif c_flag in ("-u", "--update"):
                _updater()

            elif c_flag in ("-N", "--network"):
                n = _network.Network()
                if c_val in ("-h", "--help"):
                    _logxs_out(n.__doc__)
                elif c_val in ("ip", "ipaddress"):
                    n.ip()
                elif c_val in ("i", "internet"):
                    n.internet()
                elif c_val in ("s", "speed"):
                    n.internet_speed()

            elif c_flag in ("-S", "--system"):
                s = _system.System()
                if c_val in ("-h", "--help"):
                    _logxs_out(s.__doc__)
                elif c_val in ("os"):
                    _logxs_out(s.info())
                elif c_val in ("os-info"):
                    _logxs_out(s.info(all=True))

            elif c_flag in("-L", "--lxz"):
                if c_val in LXZ_SYS_ARG: 
                    assert os.uname()[-1] == 'x86_64'
                    _call_shell(cmd=SYSX_PATH + c_val)
                
                elif c_val in LXZ_HOW:
                    _call_shell(cmd=HOWX_PATH + c_val)
                
                elif c_val in LXZ_SCRIPT + LXZ_CHECK:
                    _call_shell(cmd=SCRIPTX_PATH + c_val)
                
                elif c_val == "--dk-help":
                    _call_shell(cmd=DKX_PATH + c_val)

                else:
                    _logxs_out("Not an option ({}),  -L --help for help.".format(c_val))


    except getopt.error as err:
        # output error, and return with an error code
        logxs.printf(str(err), _int=True, _err=True)

    except AssertionError:
        logxs.printf("that command is excepted to be run on 'x86_64, pc'",
                     _int=True, _err=True)
# def _save(cmd):
#     _ = DBModel(cmd[0], str(datetime.now()), cmd[1]).save()
#     db.commit()


def _call_shell(cmd):
    # result = run(cmd, shell=True, check=False,
    #              stdout=PIPE, universal_newlines=True)
    # while True:
    #     line = result.stdout.rstrip()
    #     if not line: break
    #     yield  line
    # return result.stdout[:-1] if return_ else _logxs_out(result.stdout[:-1])

    # p = Popen(cmd, stdout = PIPE,
    #     stderr = STDOUT, shell = True, bufsize=1 )
    # for line in iter(p.stdout.readline, b''):
    #     logxs.printf(line.decode("utf-8")[:-1] , _int=True, _shell=True)
    # p.stdout.close()
    # p.wait()

    run(cmd, shell=True, stderr=stderr, stdout=stdout)


def _logxs_out(msg):
    logxs.printf(msg, _int=True)


def _updater():
    _logxs_out("updating the script...")
    cmd = "find . -maxdepth 2 -name '*.py' -print0 | xargs -0 sha1sum | sort -h | sha256sum | awk '{print $1}'"
    _call_shell(cmd, return_=False)


if __name__ == "__main__":
    main(direct=False)  # this will be invoked when python -m pylaxz -N i



"""
logxs: noice print using rich.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

by Min Latt.
License: MIT, see LICENSE for more details.
"""

from rich import print as _p

# import keyword


def printf(*argv, _int=False, _err=False, _shell=False) -> None:
    """

    Parameters:
        in << takes any given arguments
        out >> return data back, type, also shape for ML purpose.

    Usage:
    ------
    logxs.printf(ANY)

    """
    if _int:  # for internal prints and error messages
        for _ in argv:
            _print_error(_) if _err else _print_internal(_, _shell)
    else:  # called for other purpose by user.
        data, data_type, data_shape = [], [], []
        for _ in argv:
            try:
                data.append(_)
                data_type.append(type(_))
                data_shape.append(_.shape)
            except AttributeError:
                data_shape.append(None)
            except Exception as e:
                _print_error(e)

        for _ in range(len(data)):
            _print_data(data[_])
            _print_type(data_type[_])
            if not (data_shape[_] == () or data_shape[_] == None):
                _print_shape(data_shape[_])


def _print_internal(m, _shell) -> None:
    _p("[magenta]>> {0}[/magenta]".format(m)) if _shell else _p("[magenta]>> \n{0}[/magenta]".format(m))


def _print_error(m) -> None:
    _p("[red]>> {0}[/red]".format(m))


def _print_data(m) -> None:
    _p("[italic red]{0}[/italic red]".format(m))


def _print_type(m) -> None:
    _p("[blue]{0}[/blue]".format(m))


def _print_shape(m) -> None:
    _p("shape => [red]{0}[/red]".format(m))



"""
pylaxz

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Copyright (c) 2020 Min Latt.
License: MIT, see LICENSE for more details.

A small utility for my personal use cases.
"""

from .utils.logxs import printf
from .__version__ import version
from . import orm


"""
pylaxz -c --check network -n

checking everying about network.
"""

# Standard library imports
from enum import Flag
from subprocess import call
import os
import speedtest as _speedtest

from . import logxs


class Network:
    """
    Network checker

    Arguments : (ip)address, (i)nternet, (s)peed, -h

    Examples:
      $ pylaxz -n i # for internet status
      $ pylaxz -n s # for internet speed

    """

    def __init__(self) -> None:
        self.target_host = "www.google.com"
        self.timeout = 3
        self.is_internet = False
        self.arch = True if os.name == "posix" else False

    def ip(self):
        call(["ifconfig", "-a"]) if self.arch else call(["ipconfig"])

    def internet(self, log=True):
        try:
            import httplib2 as _httplib

            conn = _httplib.HTTPConnectionWithTimeout(
                self.target_host, timeout=self.timeout
            )
        except ImportError:
            import http.client as _httplib

            conn = _httplib.HTTPConnection(self.target_host, timeout=self.timeout)
        try:
            conn.request("HEAD", "/")
            conn.close()
            self.is_internet = True

        except:
            self.is_internet = False
        if log:
            logxs.printf("Internet status : " + str(self.is_internet), _int=True)

    def internet_speed(self):
        try:
            self.internet(log=False)
            if self.is_internet:
                _st = _speedtest.Speedtest()
                logxs.printf("Getting best servers, please wait ... ", _int=True)
                _st.get_best_server()

                logxs.printf("Ping : {} ms".format(_st.results.ping), _int=True)
                logxs.printf(
                    "Download, {:.2f} MB/s".format(_st.download() / 1000000), _int=True
                )
                logxs.printf(
                    "Upload, {:.2f} MB/s".format(_st.upload() / 1000000), _int=True
                )
        except _speedtest.ConfigRetrievalError:
            logxs.printf("no internet to be checked!", _int=True)




from os import path
from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

requires = [
    'rich', 
    'speedtest-cli', 
]

info = {}
with open(path.join(here, 'pylaxz', '__version__.py'), 'r') as f:
    exec(f.read(), info)

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="py-laxz",
    version=info['version'],
    description="A small until for my needs.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/minlaxz/py-laxz",
    author="Min Latt",
    author_email="minminlaxz@gmail.com",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        'Programming Language :: Python :: 3 :: Only',
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    include_package_data=True,    
    # package_dir={'pylaxz':'pylaxz'},
    keywords='sample, setuptools, development, pylaxz',
    python_requires='>=3.5, <4',
    install_requires=requires,
    project_urls={  # Optional
        'Bug Reports': 'https://github.com/minlaxz/pylaxz/issues',
        'Say Thanks!': 'http://saythanks.io/to/minminlaxz%40gmail.com',
        'Source': "https://github.com/minlaxz/py-laxz",
    },
    entry_points={"console_scripts": ["pylaxz=pylaxz.__main__:main"]},
)


"""
pylaxz -S arguments

Arguments : 
    os
    os-info

checking everying about network.
"""

from os import uname as _uname, name as _name


class System:
    """

    For System Purposes

    Arguments : (os), (os-all), -h

    Examples:
      $ pylaxz -S os # for short OS information
      $ pylaxz -S os-info # for long description about OS

    """

    def __init__(self) -> None:
        self.arch = True if _name == "posix" else False

    @property
    def __partial(self) -> None:
        return f"{_uname()}" if self.arch else f"Not supported on Windows yet."
        # logxs.printf(os.uname() if self.type else "r u on windows ? omg!" , _int=True)

    @property
    def __all(self) -> None:
        return f"Showing all information..."

    def info(self, all=False):
        return self.__all if all else self.__partial

    # @check.setter
    # def check(self, value) -> int:
    #     return None

